/****************************************************************************
** Demonstration of fading a palette in from black or fading out to black  **
**  by Steven H Don                                                        **
**                                                                         **
** For questions, feel free to e-mail me.                                  **
**                                                                         **
**    shd@earthling.net                                                    **
**    http://shd.cjb.net                                                   **
**                                                                         **
****************************************************************************/
#include <dos.h>

#ifdef __DJGPP
  #include <sys/nearptr.h>
  unsigned char *screen;
#else
  unsigned char far *screen;
#endif

unsigned char Palette [256][3];

//Switches to VGA 320x200 256 colour
void VGAScreen ()
{
  union REGS r;
  r.h.ah = 0;
  r.h.al = 0x13;
  int86(0x10, &r, &r);
}

//Switches to text 80x25
void TextScreen ()
{
  union REGS r;
  r.h.ah = 0;
  r.h.al = 0x3;
  int86(0x10, &r, &r);
}

void ReadDAC (unsigned char DAC, unsigned char *R, unsigned char *G, unsigned char *B)
{
  /*
    This read the Red Green Blue-value from a DAC register
  */
  outportb (0x3C7, DAC);
  *R = inportb (0x3C9);
  *G = inportb (0x3C9);
  *B = inportb (0x3C9);
}

void SetDAC (unsigned char DAC, unsigned char R, unsigned char G, unsigned char B)
{
  /*
    This sets a DAC register to a specific Red Green Blue-value
  */
  outportb (0x3C8, DAC);
  outportb (0x3C9, R);
  outportb (0x3C9, G);
  outportb (0x3C9, B);
}

void BlackDACs ()
{
  /*This sets all the DAC registers to black*/
  unsigned char Counter;

  for (Counter = 0; Counter < 255; Counter++)
    SetDAC (Counter, 0, 0, 0);
}

void FadeIn ()
/*
  This fades in the palette as stored in the "Palette" array.
*/
{
  unsigned char Counter, Done;
  unsigned char Red, Green, Blue;

  do {
    //Assume fading is completed
    Done = 1;
    //Adjust all 256 DAC registers
    for (Counter = 0; Counter < 255; Counter++)
    {
      //Read the DAC register
      ReadDAC (Counter, &Red, &Green, &Blue);
      //Check to see whether Red component has reached its final level
      if (Red < Palette [Counter][0])
      {
        //if not, increase it by one...
        Red++;
        //... and note that the fade is not complete yet
        Done = 0;
      }
      //Do the same for the Green component
      if (Green < Palette [Counter][1])
      {
        Green++;
        Done = 0;
      }
      //Do the same for the Blue component
      if (Blue < Palette [Counter][2])
      {
        Blue++;
        Done = 0;
      }
      //Set the DAC register to the new values
      SetDAC (Counter, Red, Green, Blue);
    }
    //Slow down the fade
    delay (10);
  } while (!Done);
}

void FadeOut ()
/*
  This fades out the on-screen palette.
*/
{
  unsigned char Counter, Done;
  unsigned char Red, Green, Blue;

  do {
    //Assume fading is completed
    Done = 1;
    //Adjust all 256 DAC registers
    for (Counter = 0; Counter < 255; Counter++)
    {
      //Read the DAC register
      ReadDAC (Counter, &Red, &Green, &Blue);
      //Check to see whether Red component has reached 0
      if (Red)
      {
        //if not, decrease it by one...
        Red--;
        //... and note that the fade is not complete yet
        Done = 0;
      }
      //Do the same for the Green component
      if (Green)
      {
        Green--;
        Done = 0;
      }
      //Do the same for the Blue component
      if (Blue) {
        Blue--;
        Done = 0;
      }
      //Set the DAC register to the new values
      SetDAC (Counter, Red, Green, Blue);
    }
    //Slow down the fade
    delay (10);
  } while (!Done);
}

void BluePalette ()
{
  //This sets up a smooth blue palette

  unsigned char Counter;

  for (Counter = 0; Counter < 255; Counter++)
  {
    //Set up a palette...
    //No red
    Palette [Counter][0] = 0;
    //No green either
    Palette [Counter][1] = 0;
    //But increasingly bright blue
    Palette [Counter][2] = Counter >> 2;
  }
}

void Gradient ()
{
  //This displays a smooth pattern on screen, just for demonstration purposes
  unsigned int OffSet;

  for (OffSet = 0; OffSet < 63999; OffSet++)
    screen [OffSet] = OffSet % 315;
}

void main ()
{
  BluePalette ();
  //Display message
  printf ("Fading demonstration\n");
  printf ("Press any key to fade out and switch to VGA graphics mode...\n");
  //Wait for keypress
  getch();
  //Fade out - note: it also works in textmode!
  FadeOut ();
  //Switch to VGA
  VGAScreen ();
  //Set up a pointer to the vga memory at A000:0000
  #ifdef __DJGPP
    __djgpp_nearptr_enable ();
    screen = (unsigned char *) (__djgpp_conventional_base + 0xA0000);
  #else
    screen = MK_FP (0xA000, 0);
  #endif
  //Clear the on-screen palette
  BlackDACs ();
  //Set the "Palette" array to a smooth blue
  BluePalette ();
  //Display a pattern on-screen
  Gradient ();
  //Fade in the blue
  FadeIn ();
  //Wait for a keypress
  getch ();
  //Fade out again
  FadeOut ();
  //And finally, return to text
  TextScreen ();
}